/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.util

import android.content.Context
import android.content.res.Configuration
import android.content.res.Resources
import android.util.TypedValue

fun pixelToDp(pixel: Int, resources: Resources): Float {
    // 해상도 마다 다른 density 를 반환.
    var density: Float = resources.displayMetrics.density
    when {
        // mpdi  160dpi xxxhdpi 기준으로 density 값 재설정
        density.toDouble() == 1.0 -> density *= 4.0f
        // hdpi  240dpi
        density.toDouble() == 1.5 -> density *= (8 / 3).toFloat()
        // xhdpi (320dpi)
        density.toDouble() == 2.0 -> density *= 2.0f
    }
    return pixel.toFloat() / density
}

fun dpToPixel(dp: Float, resources: Resources): Int =
    TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, dp, resources.displayMetrics).toInt()

object LayoutUtils {
    var pickerWidth = 0
        private set
    var pickerHeight = 0
        private set
    var orientation = Configuration.ORIENTATION_PORTRAIT
        private set
    var isPopup = false
        private set

    fun initialize(context: Context, isPopup: Boolean) {
        this.isPopup = isPopup
        orientation = context.resources.configuration.orientation

        var statusBar = 0
        val resId = context.resources.getIdentifier("status_bar_height", "dimen", "android");
        if (resId > 0) {
            statusBar = context.resources.getDimensionPixelSize(resId);
        }

        val widthPixels = context.resources.displayMetrics.widthPixels
        val heightPixels = context.resources.displayMetrics.heightPixels - statusBar // 상태바를 높이에서 제외

        val minMargin = if (orientation == Configuration.ORIENTATION_PORTRAIT) {
            dpToPixel((2 * POPUP_MIN_MARGIN_PORTRAIT).toFloat(), context.resources)
        } else {
            dpToPixel((2 * POPUP_MIN_MARGIN_LANDSCAPE).toFloat(), context.resources)
        }
        val minMarginWidth = (widthPixels - minMargin)
        val minMarginHeight = (heightPixels - minMargin)
        val maxPopupWidth = dpToPixel(POPUP_MAX_WIDTH.toFloat(), context.resources)
        val maxPopupHeight = dpToPixel(POPUP_MAX_HEIGHT.toFloat(), context.resources)
        pickerWidth = if (!isPopup) widthPixels else {
            if (maxPopupWidth + minMargin > widthPixels) minMarginWidth else maxPopupWidth
        }
        pickerHeight = if (!isPopup) heightPixels else {
            if (maxPopupHeight + minMargin > heightPixels) minMarginHeight else maxPopupHeight
        }
    }

    private const val POPUP_MAX_WIDTH = 360 // dp
    private const val POPUP_MAX_HEIGHT = 480 // dp
    private const val POPUP_MIN_MARGIN_PORTRAIT = 30 // dp
    private const val POPUP_MIN_MARGIN_LANDSCAPE = 14 // dp
    private const val FULL_TITLE_BAR_HEIGHT = 56 // dp
    private const val POPUP_TITLE_BAR_HEIGHT = 46 // dp
}
